﻿using EsignSaasApi.Core;
using log4net;
using System;
using System.Collections.Generic;
using System.IO;
using System.Net;
using System.Text;
using System.Windows.Forms;

/********************************************************************************

** 描述: 基于WebRequest/WebResponse组合调用HTTP接口的辅助类，仅供接口调试参考
** 日期: 2022/10/26

*********************************************************************************/

namespace EsignSaasApi.Core
{
    public class HttpHelper
    {
        private static ILog Logger = LogManager.GetLogger(System.Reflection.MethodBase.GetCurrentMethod().DeclaringType);
        #region 发送常规HTTP 请求
        public static HttpRespResult DoCommHttp(string host, string url, string reqType, string data, Dictionary<string, string> headers)
        {
            return executeHttpRequest(host + url, reqType, data , headers);
        }
        #endregion


        #region 执行网络请求
        private static HttpRespResult executeHttpRequest(string url, string reqType, string data,Dictionary<string, string> headers)
        {
            
            HttpWebRequest webReq = null;
            HttpWebResponse webResp = null;
            StreamReader reader = null;
            // HTTP响应状态码
            int respStatusCode = 0;
            // HTTP响应状态的描述性信息
            string respStatusCodeMsg = null;
            // 响应信息
            string respContent = string.Empty;
            // HTTP响应结果
            HttpRespResult httpRespResult = null;
            try
            {
                //构造一个Web请求的对象
                webReq = (HttpWebRequest)WebRequest.Create(url);
                Encoding encoding = System.Text.Encoding.GetEncoding("UTF-8");

                webReq.Method = reqType;
                // 把请求数据转成“UTF-8”的字节流

                if (null != headers)
                {
                    //遍历字典
                    foreach (KeyValuePair<string, string> header in headers)
                    {
                        webReq.Headers.Add(header.Key, header.Value);
                    }
                }
                string contentType = "application/json; charset=UTF-8";
                webReq.Accept = "*/*";
                webReq.ContentType = contentType;
                // 请求将跟随的重定向响应的最大数目。 默认值为 50
                webReq.MaximumAutomaticRedirections = 50;
                webReq.AllowAutoRedirect = true;


                if ("POST".Equals(reqType) || "PUT".Equals(reqType))
                {
                    byte[] bodyBytes = Encoding.UTF8.GetBytes(data);
                    webReq.ContentLength = bodyBytes.Length;

                    // 发送请求
                    Stream stream = webReq.GetRequestStream();
                    stream.Write(bodyBytes, 0, bodyBytes.Length);
                    stream.Close();

                }



                //获取Web请求的响应内容
                webResp = (HttpWebResponse)webReq.GetResponse();
                // 获取 HTTP响应状态码
                respStatusCode = Convert.ToInt32(webResp.StatusCode);
                // 获取 HTTP响应状态的描述性信息
                respStatusCodeMsg = webResp.StatusCode.ToString();

                // 通过响应流构造一个StreamReader
                reader = new StreamReader(webResp.GetResponseStream(), Encoding.UTF8);
                // 利用 StreamReader 从头到尾读取响应内容
                respContent = reader.ReadToEnd();

                httpRespResult = new HttpRespResult();
                httpRespResult.IsNetworkSuccess = true;
                httpRespResult.NetworkMsg = "网络请求成功";
                httpRespResult.HttpStatusCode = respStatusCode;
                httpRespResult.HttpStatusCodeMsg = respStatusCodeMsg;
                httpRespResult.RespData = respContent;

            }
            catch (WebException webEx)
            {
                if (webEx.Status == WebExceptionStatus.ProtocolError)
                {
                    webResp = (HttpWebResponse)webEx.Response;
                    if (null != webResp)
                    {
                        // 获取 HTTP响应状态码
                        respStatusCode = Convert.ToInt32(webResp.StatusCode);
                        // 获取 HTTP响应状态的描述性信息
                        respStatusCodeMsg = webResp.StatusCode.ToString();

                        reader = new StreamReader(webResp.GetResponseStream(), Encoding.UTF8);
                        respContent = reader.ReadToEnd();
                        string exMsg = string.Format(">>>> 网络请求地址: {0}", url);
                        Logger.Error(exMsg);
                        exMsg = string.Format(">>>> 网络请求出现 WebException 异常: {0} - {1}  StackTrace: {2}", respStatusCode, respStatusCodeMsg, webEx.StackTrace);
                        Logger.Error(exMsg);

                        httpRespResult = new HttpRespResult();
                        httpRespResult.IsNetworkSuccess = true;
                        httpRespResult.NetworkMsg = "网络请求成功";
                        httpRespResult.HttpStatusCode = respStatusCode;
                        httpRespResult.HttpStatusCodeMsg = respStatusCodeMsg;
                        httpRespResult.RespData = respContent;
                        return httpRespResult;
                    }
                    else
                    {
                        string exStatus = webEx.Status.ToString();
                        string exMessage = webEx.Message;
                        string exMsg = string.Format(">>>> 网络请求地址: {0}", url);
                        Logger.Error(exMsg);
                        exMsg = string.Format(">>>> 网络请求出现 WebException 异常: {0} - {1}  StackTrace: {2}", exStatus, exMessage, webEx.StackTrace);
                        Logger.Error(exMsg);

                        httpRespResult = new HttpRespResult();
                        httpRespResult.IsNetworkSuccess = false;
                        httpRespResult.NetworkMsg = exMessage;
                        return httpRespResult;
                        throw webEx;
                    }
                }
                else
                {
                    string exStatus = webEx.Status.ToString();
                    string exMessage = webEx.Message;
                    string exMsg = string.Format(">>>> 网络请求地址: {0}", url);
                    Logger.Error(exMsg);
                    exMsg = string.Format(">>>> 网络请求出现 WebException 异常: {0} - {1}  StackTrace: {2}", exStatus, exMessage, webEx.StackTrace);
                    Logger.Error(exMsg);

                    httpRespResult = new HttpRespResult();
                    httpRespResult.IsNetworkSuccess = false;
                    httpRespResult.NetworkMsg = exMessage;
                    return httpRespResult;
                }
            }
            catch (Exception ex)
            {
                string exMessage = ex.Message;
                string exMsg = string.Format(">>>> 网络请求地址: {0}", url);
                Logger.Error(exMsg);
                exMsg = string.Format(">>>> 网络请求出现Exception异常: {0}  StackTrace: {1}", exMessage, ex.StackTrace);
                Logger.Error(exMsg);

                httpRespResult = new HttpRespResult();
                httpRespResult.IsNetworkSuccess = false;
                httpRespResult.NetworkMsg = exMessage;
                return httpRespResult;
            }
            finally
            {
                if (null != reader)
                {
                    reader.Close();
                }
                if (null != webResp)
                {
                    webResp.Close();
                }
                if (null != webReq)
                {
                    webReq.Abort();
                }
            }
            return httpRespResult;
        }
        #endregion

        /// <summary>
        /// 模拟PUT请求上传文件
        /// </summary>
        /// <param name="url"></param>
        /// <param name="param"></param>
        /// <returns></returns>
        public static HttpRespResult DoUploadHttp(string url, string filePath, Dictionary<string, string> uploadHeaders,string contentType)
        {
            HttpWebRequest webReq = null;
            HttpWebResponse webResp = null;
            StreamReader reader = null;
            // HTTP响应状态码
            int respStatusCode = 0;
            // HTTP响应状态的描述性信息
            string respStatusCodeMsg = null;
            // 响应信息
            string respContent = string.Empty;
            // HTTP响应结果
            HttpRespResult httpRespResult = null;

            try
            {
                //注意提交的编码 这边是需要改变的 这边默认的是Default：系统当前编码
                byte[] fileData = File2Bytes(filePath);

                // 设置提交的相关参数 
                webReq = WebRequest.Create(url) as HttpWebRequest;
                Encoding myEncoding = Encoding.UTF8;
                webReq.Method = "PUT";
                webReq.KeepAlive = false;
                webReq.AllowAutoRedirect = false;
                //遍历字典
                if (null != uploadHeaders)
                {
                    // 设置请求头
                    foreach (KeyValuePair<string, string> kvp in uploadHeaders)
                    {
                        webReq.Headers.Add(kvp.Key, kvp.Value);
                    }
                }
                webReq.ContentType = contentType;
                // 提交请求数据 
                System.IO.Stream outputStream = webReq.GetRequestStream();
                outputStream.Write(fileData, 0, fileData.Length);
                outputStream.Close();


                //获取Web请求的响应内容
                webResp = (HttpWebResponse)webReq.GetResponse();
                // 获取 HTTP响应状态码
                respStatusCode = Convert.ToInt32(webResp.StatusCode);
                // 获取 HTTP响应状态的描述性信息
                respStatusCodeMsg = webResp.StatusCode.ToString();

                // 通过响应流构造一个StreamReader
                reader = new StreamReader(webResp.GetResponseStream(), Encoding.UTF8);
                // 利用 StreamReader 从头到尾读取响应内容
                respContent = reader.ReadToEnd();

                httpRespResult = new HttpRespResult();
                httpRespResult.IsNetworkSuccess = true;
                httpRespResult.NetworkMsg = "网络请求成功";
                httpRespResult.HttpStatusCode = respStatusCode;
                httpRespResult.HttpStatusCodeMsg = respStatusCodeMsg;
                httpRespResult.RespData = respContent;
            }

            catch (WebException webEx)
            {
                if (webEx.Status == WebExceptionStatus.ProtocolError)
                {
                    webResp = (HttpWebResponse)webEx.Response;
                    if (null != webResp)
                    {
                        // 获取 HTTP响应状态码
                        respStatusCode = Convert.ToInt32(webResp.StatusCode);
                        // 获取 HTTP响应状态的描述性信息
                        respStatusCodeMsg = webResp.StatusCode.ToString();

                        reader = new StreamReader(webResp.GetResponseStream(), Encoding.UTF8);
                        respContent = reader.ReadToEnd();
                        string exMsg = string.Format(">>>> 网络请求地址: {0}", url);
                        Logger.Error(exMsg);
                        exMsg = string.Format(">>>> 网络请求出现 WebException 异常: {0} - {1}  StackTrace: {2}", respStatusCode, respStatusCodeMsg, webEx.StackTrace);
                        Logger.Error(exMsg);

                        httpRespResult = new HttpRespResult();
                        httpRespResult.IsNetworkSuccess = true;
                        httpRespResult.NetworkMsg = "网络请求成功";
                        httpRespResult.HttpStatusCode = respStatusCode;
                        httpRespResult.HttpStatusCodeMsg = respStatusCodeMsg;
                        httpRespResult.RespData = respContent;
                        return httpRespResult;
                    }
                    else
                    {
                        string exStatus = webEx.Status.ToString();
                        string exMessage = webEx.Message;
                        string exMsg = string.Format(">>>> 网络请求地址: {0}", url);
                        Logger.Error(exMsg);
                        exMsg = string.Format(">>>> 网络请求出现 WebException 异常: {0} - {1}  StackTrace: {2}", exStatus, exMessage, webEx.StackTrace);
                        Logger.Error(exMsg);

                        httpRespResult = new HttpRespResult();
                        httpRespResult.IsNetworkSuccess = false;
                        httpRespResult.NetworkMsg = exMessage;
                        return httpRespResult;
                        throw webEx;
                    }
                }
                else
                {
                    string exStatus = webEx.Status.ToString();
                    string exMessage = webEx.Message;
                    string exMsg = string.Format(">>>> 网络请求地址: {0}", url);
                    Logger.Error(exMsg);
                    exMsg = string.Format(">>>> 网络请求出现 WebException 异常: {0} - {1}  StackTrace: {2}", exStatus, exMessage, webEx.StackTrace);
                    Logger.Error(exMsg);

                    httpRespResult = new HttpRespResult();
                    httpRespResult.IsNetworkSuccess = false;
                    httpRespResult.NetworkMsg = exMessage;
                    return httpRespResult;
                }
            }
            catch (Exception ex)
            {
                string exMessage = ex.Message;
                string exMsg = string.Format(">>>> 网络请求地址: {0}", url);
                Logger.Error(exMsg);
                exMsg = string.Format(">>>> 网络请求出现Exception异常: {0}  StackTrace: {1}", exMessage, ex.StackTrace);
                Logger.Error(exMsg);

                httpRespResult = new HttpRespResult();
                httpRespResult.IsNetworkSuccess = false;
                httpRespResult.NetworkMsg = exMessage;
                return httpRespResult;
            }
            finally
            {
                if (null != reader)
                {
                    reader.Close();
                }
                if (null != webResp)
                {
                    webResp.Close();
                }
                if (null != webReq)
                {
                    webReq.Abort();
                }
            }
            return httpRespResult;
        }

        private static byte[] File2Bytes(string filePath)
        {
            if (!System.IO.File.Exists(filePath))
            {
                return new byte[0];
            }

            FileInfo fi = new FileInfo(filePath);
            byte[] buff = new byte[fi.Length];

            FileStream fs = fi.OpenRead();
            fs.Read(buff, 0, Convert.ToInt32(fs.Length));
            fs.Close();

            return buff;
        }



        /**
         * @description 创建文件流上传 请求头
         *
         * @param fileContentMd5
         * @param contentType
         * @return
         * @author 澄泓
         */
        public static Dictionary<string, string> BuildUploadHeader(String fileContentMd5)
        {
            Dictionary<string, string> header = new Dictionary<string, string>();
            header.Add("Content-MD5", fileContentMd5);
            return header;
        }



        //构造json+签名鉴权请求头
        public static Dictionary<string, string> BuildSignAndJsonHeader(string projectId, string contentMD5, string accept, string contentType, string authMode)
        {

            Dictionary < string, string> header = new Dictionary<string, string>();
            header.Add("X-Tsign-Open-App-Id", projectId);
            header.Add("X-Tsign-Open-Ca-Timestamp", Convert.ToString(TimestampHelper.GetTimestamp()));
            header.Add("Content-MD5", contentMD5);
            header.Add("X-Tsign-Open-Auth-Mode", authMode);
            return header;
        }

        /**
         * 签名计算并且构建一个签名鉴权+json数据的esign请求头
         * @param  httpMethod
         *      *         The name of a supported {@linkplain java.nio.charset.Charset
         *      *         charset}
         * @return
         */
        public static Dictionary<string, string> SignAndBuildSignAndJsonHeader(string appid, string secret, string reqData, string reqType, string url)
        {
            string contentMD5="";
            if ("GET".Equals(reqType))
            {
                reqData = null;
                contentMD5 = "";
            }
            else 
            {
                //对body体做md5摘要
                contentMD5 = DigestHelper.doContentMD5(reqData);
            }
            string contentType = "application/json; charset=UTF-8";
            string Mode = "Signature";
            string Accept = "*/*";


            //构造一个初步的请求头
            Dictionary<string, string> esignHeaderMap = BuildSignAndJsonHeader(appid, contentMD5, Accept, contentType, Mode);

            //传入生成的bodyMd5,加上其他请求头部信息拼接成字符串
            string stringToSign = DigestHelper.AppendSignDatastring(reqType, esignHeaderMap["Content-MD5"], Accept, contentType, "","", url);
            //整体做sha256签名
            string reqSignature = DigestHelper.DoSignatureBase64(stringToSign, secret);
            //请求头添加签名值
            esignHeaderMap.Add("X-Tsign-Open-Ca-Signature", reqSignature);
      
            return esignHeaderMap;
            }
        }

}